@@ -38,8 +38,10 @@ def parse_http_success(response):
     if response.request.method in ['GET']:
         reason = HTTP_RESPONSE_CODES[response.status_code]
         error = ''
-        if response.json():
+        if 'json' in response.headers.get('Content-Type'):
             json_response = response.json()
+        elif 'text' in response.headers.get('Content-Type'):
+            json_response = response.text
         else:
             json_response = dict()
             reason = HTTP_RESPONSE_CODES[response.status_code]
@@ -108,7 +110,7 @@ def parse_response(response):
 
 class Silverpeak(object):
     def __init__(self, user, user_pass, sp_server, sp_port="443",
-                 verify=False, disable_warnings=False, timeout=10, auto_login=True):
+                 verify=False, disable_warnings=False, proxies=None, timeout=10, auto_login=True):
         self.user = user
         self.user_pass = user_pass
         self.sp_server = sp_server
@@ -128,6 +130,9 @@ class Silverpeak(object):
 
         self.session = requests.session()
 
+        if proxies is not None:
+            self.session.proxies = proxies
+
         if not self.verify:
             self.session.verify = self.verify
 
@@ -337,7 +342,7 @@ class Silverpeak(object):
 
     def get_discovered(self):
         """
-        Reurns all the discovered appliances
+        Returns all the discovered appliances
         :return: Result named tuple
         """
         url = '{}/appliance/discovered'.format(self.base_url)
@@ -346,7 +351,7 @@ class Silverpeak(object):
 
     def get_approved(self):
         """
-        Reurns all approved appliances
+        Returns all approved appliances
         :return: Result named tuple
         """
         url = '{}/appliance/approved'.format(self.base_url)
@@ -355,7 +360,7 @@ class Silverpeak(object):
 
     def get_denied(self):
         """
-        Reurns all the denied appliances
+        Returns all the denied appliances
         :return: Result named tuple
         """
         url = '{}/appliance/denied'.format(self.base_url)
@@ -364,7 +369,7 @@ class Silverpeak(object):
 
     def get_interfaces(self, applianceID, cashed='true'):
         """
-        Reurns node configuration data from orchestrator database or from the specified appliance
+        Returns node configuration data from orchestrator database or from the specified appliance
         :param applianceID: The node ID of the appliance
         :param cashed: True/false Get from orchestrator/get from appliance
         :return: Result named tuple
@@ -375,7 +380,7 @@ class Silverpeak(object):
         return self._get(self.session, url)
 
     def get_device_alarms(self, applianceID, view='all',
-                          severity='', order='', maxAlarms=5):
+                          severity='', order='', maxAlarms=5, start_time=None, end_time=None):
         """
         Returns active, historical, or all alarms for appliances whos id's are provided in the request body
         :param applianceID: The node ID of the appliance
@@ -383,6 +388,8 @@ class Silverpeak(object):
         :param severity: Filters alarms by severity (warning, minor, major, critical)
         :param order: Order by alarm severity (true, false)
         :param maxAlarms: How many alarms to show (default=5)
+        :param start_time: EPOCH Time
+        :param end_time: EPOCH Time
         :return: Result named tuple
         """
         url = '{}/alarm/appliance?view={}&maxAlarms={}'.format(
@@ -394,6 +401,12 @@ class Silverpeak(object):
         if order:
             url = '{}&order={}'.format(url, order)
 
+        if start_time:
+            url = '{}&to={}'.format(url, start_time)
+
+        if end_time:
+            url = '{}&from={}'.format(url, end_time)
+
         return self._post(
             session=self.session,
             url=url,
@@ -404,8 +417,8 @@ class Silverpeak(object):
 
     def get_alarms(self, view='all', severity=''):
         """
-        Reurns active, historical, or all alarms for appliances whos id's are provided in the request body
-        :param view: Filters arams by active, closed, all
+        Returns active, historical, or all alarms for appliances whos id's are provided in the request body
+        :param view: Filters alarms by active, closed, all
         :param severity: Filters alarms by severity (warning, minor, major, critical)
         :return: Result named tuple
         """
@@ -484,9 +497,11 @@ class Silverpeak(object):
         url = '{}/appliance/rest/{}/reboot'.format(self.base_url, applianceID)
 
         if factoryReset is None:
-            data = '{"reboot_type":"Normal","save_db":true,"clear_nm":false,"next_partition":false,"empty_db":false,"empty_db_err":false,"delay":0}'
+            data = '{"reboot_type":"Normal","save_db":true,"clear_nm":false,' \
+                   '"next_partition":false,"empty_db":false,"empty_db_err":false,"delay":0}'
         else:
-            data = '{"reboot_type":"Normal","save_db":true,"clear_nm":false,"next_partition":false,"empty_db":false,"empty_db_err":false,"delay":0,"reset_factory":true,"support_bypass":false}'
+            data = '{"reboot_type":"Normal","save_db":true,"clear_nm":false,"next_partition":false,' \
+                   '"empty_db":false,"empty_db_err":false,"delay":0,"reset_factory":true,"support_bypass":false}'
 
         return self._post(
             session=self.session,
@@ -501,7 +516,6 @@ class Silverpeak(object):
         """
         Configure Boost on an appliance
         :param applianceID: The node ID of the appliance
-        :param mini: enable or disable the mini license
         :param plus: enable or disable the plus license
         :param boost: enable or disable the boost license
         :param boostBandwidth: choose bandwidth to boost by
@@ -663,6 +677,7 @@ class Silverpeak(object):
         """
         Update deployment config of appliance
         :param applianceID: The node ID of the appliance
+        :param deploymentData: Deployment configuration data
         :return: Result named tuple
         """
         url = '{}/appliance/rest/{}/deployment'.format(
@@ -706,3 +721,99 @@ class Silverpeak(object):
             headers={'Content-Type': 'application/json'},
             data=portForwardingData,
         )
+
+    def get_interface_labels(self, active=None, labelType=None):
+        """
+        Get all the interface labels saved
+        :param labelType: Optional, Type of interface that you want to retrieve the list of labels of [wan, lan]
+        :param active: Optional, Boolean flag to return only the active (active = true) or
+        only inactive (active = false)
+        :return: Result named tuple
+        """
+
+        url = '{}/gms/interfaceLabels'.format(self.base_url)
+
+        if labelType is not None:
+            url += '/' + labelType
+
+        if active is not None:
+            url += '?active=' + str(active).lower()
+
+        return self._get(self.session, url)
+
+    def post_interface_labels(self, interfaceLabelsData, deleteDependencies=None):
+        """
+        Save interface labels, completely replacing the current implementation.
+        You cannot remove labels that are in use in an overlay
+        :param interfaceLabelsData: Object of labels (in json) to save
+        (will overwrite the current lan labels list). To remove a label, set the 'active' to false
+        :param deleteDependencies: Boolean flag whether or not you want to delete the labels from port profiles
+        and templates using it.
+        :return: Result named tuple
+        """
+
+        url = '{}/gms/interfaceLabels'.format(self.base_url)
+
+        if deleteDependencies is not None:
+            if deleteDependencies:
+                url += '?deleteDependencies=true'
+            else:
+                url += '?deleteDependencies=false'
+
+        return self._post(
+            session=self.session,
+            url=url,
+            headers={'Content-Type': 'application/json'},
+            data=interfaceLabelsData,
+            timeout=self.timeout
+        )
+
+    def get_gms_server_info(self):
+        """
+        Get orchestrator server information such as used disk space, hostname, release, etc...
+        :return: Result named tuple
+        """
+
+        url = '{}/gmsserver/info'.format(self.base_url)
+
+        return self._get(self.session, url)
+
+    def get_gms_server_brief_info(self):
+        """
+        Get orchestrator server information such as used disk space, hostname, release, etc...
+        :return: Result named tuple
+        """
+
+        url = '{}/gmsserver/briefInfo'.format(self.base_url)
+
+        return self._get(self.session, url)
+
+    def get_gms_versions(self):
+        """
+        Get available orchestrator versions
+        :return: Result named tuple
+        """
+
+        url = '{}/gms/versions'.format(self.base_url)
+
+        return self._get(self.session, url)
+
+    def get_gms_operating_system(self):
+        """
+        Get orchestrator operating system type
+        :return: Result named tuple
+        """
+
+        url = '{}/gmsOperatingSystem'.format(self.base_url)
+
+        return self._get(self.session, url)
+
+    def get_gms_server_to_say_hello(self):
+        """
+        Get hello message
+        :return: Result named tuple
+        """
+
+        url = '{}/gmsserver/hello'.format(self.base_url)
+
+        return self._get(self.session, url)
